@@ -51,6 +51,7 @@
 #include <gpgme.h>
 
 #include <istream>
+#include <numeric>
 #ifndef NDEBUG
 #include <iostream>
 using std::cerr;
@@ -563,44 +564,66 @@ KeyGenerationResult Context::keyGenerationResult() const
     }
 }
 
-Error Context::exportPublicKeys(const char *pattern, Data &keyData)
+Error Context::exportPublicKeys(const char *pattern, Data &keyData, unsigned int flags)
 {
     d->lastop = Private::Export;
     Data::Private *const dp = keyData.impl();
-    return Error(d->lasterr = gpgme_op_export(d->ctx, pattern, 0, dp ? dp->data : nullptr));
+    return Error(d->lasterr = gpgme_op_export(d->ctx, pattern, flags, dp ? dp->data : nullptr));
+}
+
+Error Context::exportPublicKeys(const char *patterns[], Data &keyData, unsigned int flags)
+{
+    d->lastop = Private::Export;
+#ifndef HAVE_GPGME_EXT_KEYLIST_MODE_EXTERNAL_NONBROKEN
+    if (!patterns || !patterns[0] || !patterns[1]) {
+        // max. one pattern -> use the non-ext version
+        return exportPublicKeys(patterns ? patterns[0] : nullptr, keyData, flags);
+    }
+#endif
+    Data::Private *const dp = keyData.impl();
+    return Error(d->lasterr = gpgme_op_export_ext(d->ctx, patterns, flags, dp ? dp->data : nullptr));
+}
+
+Error Context::startPublicKeyExport(const char *pattern, Data &keyData, unsigned int flags)
+{
+    d->lastop = Private::Export;
+    Data::Private *const dp = keyData.impl();
+    return Error(d->lasterr = gpgme_op_export_start(d->ctx, pattern, flags, dp ? dp->data : nullptr));
+}
+
+Error Context::startPublicKeyExport(const char *patterns[], Data &keyData, unsigned int flags)
+{
+    d->lastop = Private::Export;
+#ifndef HAVE_GPGME_EXT_KEYLIST_MODE_EXTERNAL_NONBROKEN
+    if (!patterns || !patterns[0] || !patterns[1]) {
+        // max. one pattern -> use the non-ext version
+        return startPublicKeyExport(patterns ? patterns[0] : nullptr, keyData, flags);
+    }
+#endif
+    Data::Private *const dp = keyData.impl();
+    return Error(d->lasterr = gpgme_op_export_ext_start(d->ctx, patterns, flags, dp ? dp->data : nullptr));
+}
+
+
+/* Same as above but without flags  */
+Error Context::exportPublicKeys(const char *pattern, Data &keyData)
+{
+    return exportPublicKeys(pattern, keyData, 0);
 }
 
 Error Context::exportPublicKeys(const char *patterns[], Data &keyData)
 {
-    d->lastop = Private::Export;
-#ifndef HAVE_GPGME_EXT_KEYLIST_MODE_EXTERNAL_NONBROKEN
-    if (!patterns || !patterns[0] || !patterns[1]) {
-        // max. one pattern -> use the non-ext version
-        return exportPublicKeys(patterns ? patterns[0] : nullptr, keyData);
-    }
-#endif
-    Data::Private *const dp = keyData.impl();
-    return Error(d->lasterr = gpgme_op_export_ext(d->ctx, patterns, 0, dp ? dp->data : nullptr));
+    return exportPublicKeys(patterns, keyData, 0);
 }
 
 Error Context::startPublicKeyExport(const char *pattern, Data &keyData)
 {
-    d->lastop = Private::Export;
-    Data::Private *const dp = keyData.impl();
-    return Error(d->lasterr = gpgme_op_export_start(d->ctx, pattern, 0, dp ? dp->data : nullptr));
+    return startPublicKeyExport(pattern, keyData, 0);
 }
 
 Error Context::startPublicKeyExport(const char *patterns[], Data &keyData)
 {
-    d->lastop = Private::Export;
-#ifndef HAVE_GPGME_EXT_KEYLIST_MODE_EXTERNAL_NONBROKEN
-    if (!patterns || !patterns[0] || !patterns[1]) {
-        // max. one pattern -> use the non-ext version
-        return startPublicKeyExport(patterns ? patterns[0] : nullptr, keyData);
-    }
-#endif
-    Data::Private *const dp = keyData.impl();
-    return Error(d->lasterr = gpgme_op_export_ext_start(d->ctx, patterns, 0, dp ? dp->data : nullptr));
+    return startPublicKeyExport(patterns, keyData, 0);
 }
 
 ImportResult Context::importKeys(const Data &data)
@@ -1320,6 +1343,11 @@ Error Context::cancelPendingOperation()
     return Error(gpgme_cancel_async(d->ctx));
 }
 
+Error Context::cancelPendingOperationImmediately()
+{
+    return Error(gpgme_cancel(d->ctx));
+}
+
 bool Context::poll()
 {
     gpgme_error_t e = GPG_ERR_NO_ERROR;
@@ -1504,6 +1532,102 @@ Error Context::startCreateSubkey(const Key &k, const char *algo,
                  k.impl(), algo, reserved, expires, flags));
 }
 
+static std::string getLFSeparatedListOfStrings(const std::vector<std::string> &strings)
+{
+    if (strings.empty()) {
+        return std::string();
+    }
+
+    return std::accumulate(
+        std::next(strings.begin()),
+        strings.end(),
+        strings[0],
+        [](const std::string &a, const std::string &b) {
+            return a + '\n' + b;
+        }
+    );
+}
+
+static std::string getLFSeparatedListOfFingerprintsFromSubkeys(const std::vector<Subkey> &subkeys)
+{
+    if (subkeys.empty()) {
+        return std::string();
+    }
+
+    std::vector<std::string> fprs;
+    fprs.reserve(subkeys.size());
+    for (auto &it : subkeys) {
+        if (it.fingerprint()) {
+            fprs.push_back(std::string(it.fingerprint()));
+        }
+    }
+
+    return getLFSeparatedListOfStrings(fprs);
+}
+
+Error Context::setExpire(const Key &k, unsigned long expires,
+                         const std::vector<Subkey> &subkeys,
+                         const Context::SetExpireFlags flags)
+{
+    std::string subfprs;
+    if (flags & Context::SetExpireAllSubkeys) {
+        subfprs = "*";
+    } else {
+        subfprs = getLFSeparatedListOfFingerprintsFromSubkeys(subkeys);
+    }
+    return Error(d->lasterr = gpgme_op_setexpire(d->ctx,
+                 k.impl(), expires, subfprs.c_str(), 0));
+}
+
+Error Context::startSetExpire(const Key &k, unsigned long expires,
+                              const std::vector<Subkey> &subkeys,
+                              const Context::SetExpireFlags flags)
+{
+    std::string subfprs;
+    if (flags & Context::SetExpireAllSubkeys) {
+        subfprs = "*";
+    } else {
+        subfprs = getLFSeparatedListOfFingerprintsFromSubkeys(subkeys);
+    }
+    return Error(d->lasterr = gpgme_op_setexpire_start(d->ctx,
+                 k.impl(), expires, subfprs.c_str(), 0));
+}
+
+static std::string getLFSeparatedListOfUserIds(const std::vector<UserID> &userIds)
+{
+    if (userIds.empty()) {
+        return std::string();
+    }
+
+    std::vector<std::string> uids;
+    uids.reserve(userIds.size());
+    for (auto &userId : userIds) {
+        if (userId.id()) {
+            uids.push_back(std::string(userId.id()));
+        }
+    }
+
+    return getLFSeparatedListOfStrings(uids);
+}
+
+Error Context::revokeSignature(const Key &key, const Key &signingKey,
+                               const std::vector<UserID> &userIds)
+{
+    const unsigned int flags = userIds.size() > 1 ? GPGME_REVSIG_LFSEP : 0;
+    const std::string uids = getLFSeparatedListOfUserIds(userIds);
+    return Error(d->lasterr = gpgme_op_revsig(d->ctx,
+                 key.impl(), signingKey.impl(), uids.c_str(), flags));
+}
+
+Error Context::startRevokeSignature(const Key &key, const Key &signingKey,
+                                    const std::vector<UserID> &userIds)
+{
+    const unsigned int flags = userIds.size() > 1 ? GPGME_REVSIG_LFSEP : 0;
+    const std::string uids = getLFSeparatedListOfUserIds(userIds);
+    return Error(d->lasterr = gpgme_op_revsig_start(d->ctx,
+                 key.impl(), signingKey.impl(), uids.c_str(), flags));
+}
+
 Error Context::setFlag(const char *name, const char *value)
 {
   return Error(d->lasterr = gpgme_set_ctx_flag(d->ctx, name, value));
@@ -1615,6 +1739,8 @@ std::ostream &operator<<(std::ostream &os, KeyListMode mode)
     CHECK(Validate);
     CHECK(Ephemeral);
     CHECK(WithTofu);
+    CHECK(WithKeygrip);
+    CHECK(WithSecret);
 #undef CHECK
     return os << ')';
 }
