@@ -78,8 +78,7 @@ void _gpgme_engine_set_status_handler (engine_t engine,
 				       void *fnc_value);
 gpgme_error_t _gpgme_engine_set_command_handler (engine_t engine,
 						 engine_command_handler_t fnc,
-						 void *fnc_value,
-						 gpgme_data_t data);
+						 void *fnc_value);
 gpgme_error_t
 _gpgme_engine_set_colon_line_handler (engine_t engine,
 				      engine_colon_line_handler_t fnc,
@@ -132,6 +131,11 @@ gpgme_error_t _gpgme_engine_op_keysign (engine_t engine,
                                         unsigned long expires,
                                         unsigned int flags,
                                         gpgme_ctx_t ctx);
+gpgme_error_t _gpgme_engine_op_revsig (engine_t engine,
+                                       gpgme_key_t key,
+                                       gpgme_key_t signing_key,
+                                       const char *userid,
+                                       unsigned int flags);
 gpgme_error_t _gpgme_engine_op_tofu_policy (engine_t engine,
                                             gpgme_key_t key,
                                             gpgme_tofu_policy_t policy);
@@ -211,6 +215,11 @@ gpgme_error_t _gpgme_engine_op_spawn (engine_t engine,
                                       gpgme_data_t dataout,
                                       gpgme_data_t dataerr,
                                       unsigned int flags);
+gpgme_error_t _gpgme_engine_op_setexpire (engine_t engine,
+                                          gpgme_key_t key,
+                                          unsigned long expires,
+                                          const char *subfprs,
+                                          unsigned int reserved);
 
 /* The available engine option flags.  */
 #define GPGME_ENGINE_FLAG_OFFLINE        1
