@@ -253,6 +253,7 @@ gpgme_release (gpgme_ctx_t ctx)
   free (ctx->request_origin);
   free (ctx->auto_key_locate);
   free (ctx->trust_model);
+  free (ctx->cert_expire);
   _gpgme_engine_info_release (ctx->engine_info);
   ctx->engine_info = NULL;
   DESTROY_LOCK (ctx->lock);
@@ -533,6 +534,14 @@ gpgme_set_ctx_flag (gpgme_ctx_t ctx, const char *name, const char *value)
       if (!ctx->override_session_key)
         err = gpg_error_from_syserror ();
     }
+  else if (!strcmp (name, "include-key-block"))
+    {
+      ctx->include_key_block = abool;
+    }
+  else if (!strcmp (name, "auto-key-import"))
+    {
+      ctx->auto_key_import = abool;
+    }
   else if (!strcmp (name, "auto-key-retrieve"))
     {
       ctx->auto_key_retrieve = abool;
@@ -566,6 +575,17 @@ gpgme_set_ctx_flag (gpgme_ctx_t ctx, const char *name, const char *value)
       if (!ctx->trust_model)
         err = gpg_error_from_syserror ();
     }
+  else if (!strcmp (name, "extended-edit"))
+    {
+      ctx->extended_edit = abool;
+    }
+  else if (!strcmp (name, "cert-expire"))
+    {
+      free (ctx->cert_expire);
+      ctx->cert_expire = strdup (value);
+      if (!ctx->cert_expire)
+        err = gpg_error_from_syserror ();
+    }
   else
     err = gpg_error (GPG_ERR_UNKNOWN_NAME);
 
@@ -603,6 +623,14 @@ gpgme_get_ctx_flag (gpgme_ctx_t ctx, const char *name)
     {
       return ctx->override_session_key? ctx->override_session_key : "";
     }
+  else if (!strcmp (name, "include-key-block"))
+    {
+      return ctx->include_key_block? "1":"";
+    }
+  else if (!strcmp (name, "auto-key-import"))
+    {
+      return ctx->auto_key_import? "1":"";
+    }
   else if (!strcmp (name, "auto-key-retrieve"))
     {
       return ctx->auto_key_retrieve? "1":"";
@@ -623,6 +651,14 @@ gpgme_get_ctx_flag (gpgme_ctx_t ctx, const char *name)
     {
       return ctx->auto_key_locate? ctx->auto_key_locate : "";
     }
+  else if (!strcmp (name, "extended-edit"))
+    {
+      return ctx->extended_edit ? "1":"";
+    }
+  else if (!strcmp (name, "cert-expire"))
+    {
+      return ctx->cert_expire? ctx->cert_expire : "";
+    }
   else
     return NULL;
 }
